<?php

namespace FiloBlu\Refilo\Remote\Entity;

use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product\Attribute\Source\Status as AttributeStatus;
use Magento\Catalog\Model\Product\Visibility;
use Magento\ConfigurableProduct\Api\Data\OptionInterface;

/**
 * Class MagentoToRefiloProductAdapter
 * @package FiloBlu\Refilo\Remote\Entity
 */
class MagentoToRefiloProductAdapter
{
    /**
     * @var \FiloBlu\Refilo\Helper\Catalog\ProductAttributeHelper
     */
    private $productAttributeHelper;

    /**
     * @var \FiloBlu\Refilo\Helper\Catalog\ProductHelper
     */
    private $productHelper;

    /**
     * @var \Magento\Catalog\Model\Product\Option
     */
    private $option;

    /**
     * MagentoToRefiloProductAdapter constructor.
     * @param \FiloBlu\Refilo\Helper\Catalog\ProductAttributeHelper $productAttributeHelper
     * @param \FiloBlu\Refilo\Helper\Catalog\ProductHelper $productHelper
     * @param \Magento\Catalog\Model\Product\Option $option
     */
    public function __construct(
        \FiloBlu\Refilo\Helper\Catalog\ProductAttributeHelper $productAttributeHelper,
        \FiloBlu\Refilo\Helper\Catalog\ProductHelper $productHelper,
        \Magento\Catalog\Model\Product\Option $option
    )
    {
        $this->productAttributeHelper = $productAttributeHelper;
        $this->productHelper = $productHelper;
        $this->option = $option;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function adapt(ProductInterface $magentoProduct, Product $refiloProduct): Product
    {
        $docType = Product::DOCTYPE_SPEC_DISABLED;
        $visibility = '0';
        $searchVisibility = '0';

        if ((int)$magentoProduct->getStatus() === AttributeStatus::STATUS_ENABLED) {
            $docType = Product::DOCTYPE_SPEC_ENABLED;
            $visibility = (!$this->isProductVisible($magentoProduct) ? '0' : '1');
            $searchVisibility = (!$this->isProductVisibleInSearch($magentoProduct) ? '0' : '1');
        }

        $data = [
            Product::DOCTYPE           => $docType,
            Product::ID                => (int)$magentoProduct->getEntityId(),
            Product::ENABLED           => ((int)$magentoProduct->getStatus() === AttributeStatus::STATUS_ENABLED) ? '1' : '0',
            Product::SKU               => $magentoProduct->getSku(),
            Product::TYPE              => $magentoProduct->getTypeId(),
            Product::NAME              => $magentoProduct->getName(),
            Product::CREATED_AT        => $magentoProduct->getCreatedAt(),
            Product::UPDATED_AT        => $magentoProduct->getUpdatedAt(),
            Product::WEIGHT            => $magentoProduct->getWeight(),
            Product::META_TITLE        => $magentoProduct->getMetaTitle(),
            Product::META_KEYWORDS     => $magentoProduct->getMetaKeywords(),
            Product::META_DESCRIPTION  => $magentoProduct->getMetaDescription(),
            Product::VISIBLE           => $visibility,
            Product::ATTRIBUTES        => $this->productAttributeHelper->getAttributesValues($magentoProduct),
            Product::FILTERS           => $this->productAttributeHelper->getFilterableAttributes($magentoProduct),
            Product::SORTABLES         => $this->productAttributeHelper->getSortableAttributes($magentoProduct),
            Product::UPSELL            => $magentoProduct->getUpSellProductIds(),
            Product::CROSSSELL         => $magentoProduct->getCrossSellProductIds(),
            Product::RELATED           => $magentoProduct->getRelatedProductIds(),
            Product::ASSETS            => $this->getAssets($magentoProduct),
            Product::INVENTORY         => $this->productHelper->getStockFor($magentoProduct),
            Product::SEARCH_VISIBILITY => $searchVisibility,
            Product::CUSTOM_OPTIONS    => $this->getCustomOptions($magentoProduct)
        ];


        $prices = $this->productHelper->getPricesFor($magentoProduct);
        $refiloProduct->setData(array_merge($data, $prices));
        $refiloProduct->setData($data);
        $this->fillChildren($magentoProduct, $refiloProduct);

        return $refiloProduct;
    }

    /**
     * @param $magentoProduct
     * @return bool
     */
    public function isProductVisible($magentoProduct)
    {
        switch ((int)$magentoProduct->getVisibility()) {
            case Visibility::VISIBILITY_IN_CATALOG:
            case Visibility::VISIBILITY_BOTH:
                return true;
        }

        return false;
    }

    /**
     * @param $magentoProduct
     * @return bool
     */
    public function isProductVisibleInSearch($magentoProduct)
    {
        switch ((int)$magentoProduct->getVisibility()) {
            case Visibility::VISIBILITY_IN_SEARCH:
            case Visibility::VISIBILITY_BOTH:
                return true;
        }

        return false;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @return array
     */
    protected function getAssets(ProductInterface $magentoProduct): array
    {
        $assets = [];
        $gallery = $magentoProduct->getData('media_gallery');

        foreach ($gallery['images'] as $image) {
            // Skip image is hide
            if (isset($image['disabled']) && $image['disabled'] == 1) {
                continue;
            }
            $assets['images'][] = [
                'label'             => $image['label'] ?? null, // TODO : remove?
                'alt'               => $image['label'] ?? null,
                'file'              => ($image['file']) ? "/catalog/product{$image['file']}" : null,
                'media_type'        => $image['media_type'] ?? null,
                'video_url'         => $image['video_url'] ?? null,
                'video_title'       => $image['video_title'] ?? null,
                'video_description' => $image['video_description'] ?? null,
                'video_metadata'    => $image['video_metadata'] ?? null,
                'video_provider'    => $image['video_provider'] ?? null,
            ];
        }

        $assets['placeholders'] = $this->productAttributeHelper->getImagePlaceholdersAttributes($magentoProduct);

        return $assets;
    }

    /**
     * @param ProductInterface $magentoProduct
     */
    public function getCustomOptions(ProductInterface $magentoProduct)
    {
        $customOptions = [];

        // Do not call following collection because memory leak
        // $collection = $magentoProduct->getProductOptionsCollection();

        /** @var \Magento\Catalog\Model\ResourceModel\Product\Option $item */
        foreach ($this->option->getProductOptionCollection($magentoProduct)->getItems() as $item) {
            $customOptions[] = $item->getData();
        }

        return $customOptions;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    protected function fillChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        switch ($magentoProduct->getTypeId()) {
            case 'configurable':
                return $this->getConfigurableChildren($magentoProduct, $refiloProduct);
            case 'bundle':
                return $this->getBundleChildren($magentoProduct, $refiloProduct);
            case 'grouped':
                return $this->getGroupedChildren($magentoProduct, $refiloProduct);
            case 'giftcard':
                return $this->getGiftCardsChildren($magentoProduct, $refiloProduct);
        }

        return $refiloProduct;
    }

    /**
     * @param ProductInterface|\Magento\ConfigurableProduct\Model\Product\Type\Configurable $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getGiftCardsChildren(ProductInterface $magentoProduct, Product $refiloProduct): Product
    {
        $giftCardsAmounts = [];
        $webSiteId = $magentoProduct->getStore()->getWebsite()->getId();
        $hasWebsiteValue = false;

        foreach ( $magentoProduct->getGiftcardAmounts() as $amount) {
           if(!$hasWebsiteValue && (int)$amount['website_id'] > 0) {
               $hasWebsiteValue = true; break;
           }
        }

        foreach ($magentoProduct->getGiftcardAmounts() as  $data)
        {
            if($hasWebsiteValue && (int)$data['website_id'] === 0 ) {
                continue;
            }

            $giftCardsAmounts[] = $data;
        }

        $attributes =  $refiloProduct->getData('attributes');
        $attributes['giftcard_amounts'] = [
                'title' => 'Amount',
                'value' => $giftCardsAmounts
        ];

        $refiloProduct->setData('attributes', $attributes);
        return $refiloProduct;
    }


    /**
     * @param ProductInterface|\Magento\ConfigurableProduct\Model\Product\Type\Configurable $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getConfigurableChildren(ProductInterface $magentoProduct, Product $refiloProduct): Product
    {
        if ($magentoProduct->getTypeId() === 'configurable') {

            return $refiloProduct;
            $storeId = $magentoProduct->getStore()->getId();
            $extensionAttributes = $magentoProduct->getExtensionAttributes();

            /** @var \Magento\ConfigurableProduct\Model\Product\Type\Configurable $configurable */
            $configurable = $magentoProduct->getTypeInstance();
            $productsChildren = $configurable->getUsedProducts($magentoProduct);

            $chidrenIds = [];

            $sellableAttributes = [];

            $usedAttributes = $configurable->getUsedProductAttributes($magentoProduct);

            foreach ($usedAttributes as $attributeId => $attribute) {
                $sellableAttributes[$attribute->getAttributeCode()] = [];
            }

            foreach ($productsChildren as $productChild) {
                if ((int)$productChild->getStatus() !== AttributeStatus::STATUS_ENABLED) {
                    continue;
                }

                foreach ($sellableAttributes as $attributeCode => $data) {
                    $sellableAttributes[$attributeCode][$productChild->getData($attributeCode)] = true;
                }

                $chidrenIds[] = (int)$productChild->getId();
            }

            $refiloProduct->setChildren($chidrenIds);
            $productOptions = $extensionAttributes->getConfigurableProductOptions() ?? [];

            //   $magentoProduct->getConfigurableOptions($magentoProduct);

            $variants = [];
            foreach ($productOptions as $option) {

                $attributeCode = $option->getProductAttribute()->getAttributeCode();
                $variant = [
                    'id'             => $option->getId(),
                    'type'           => $option->getProductAttribute()->getData('frontend_input'),
                    'position'       => $option->getPosition(),
                    'label'          => $option->getProductAttribute()->getStoreLabel($storeId),
                    'use_default'    => (int)$option->getIsUseDefault(),
                    'attribute_id'   => $option->getAttributeId(),
                    'attribute_code' => $attributeCode,
                    'values'         => $this->getOptions($option, $sellableAttributes[$attributeCode]),
                    'options'        => []
                ];

                $additionalData = json_decode($option->getProductAttribute()->getData('additional_data'), true);

                if ($additionalData) {
                    $variant['rendered'] = isset($additionalData['swatch_input_type']) ? $additionalData['swatch_input_type'] : '';
                }

                $options = [];
                foreach ($variant['values'] as $value) {
                    $options[] = (int)$value['value_index'];
                }

                $variant['options'] = $options;
                $variants[] = $variant;
            }

            //$salableChildren = $magentoProduct->getTypeInstance()->getSalableUsedProducts($magentoProduct, null);

            $filters = [];
            foreach ($productsChildren as $salableChild) {
                $filterable = $this->productAttributeHelper->getFilterableAttributes($salableChild);

                foreach ($filterable as $attributeCode => $value) {
                    // TODO: handle cases wich not are multiselect or select
                    if (!isset($value['key']) || !$value['key']) {
                        continue;
                    }

                    if (!isset($filters[$attributeCode])) {
                        $filters[$attributeCode] = $value;
                        unset($filters[$attributeCode]['swatch']);

                        if (!is_array($value['value'])) {
                            $filters[$attributeCode]['value'] = [$value['value']];
                        }

                        if (!is_array($value['key'])) {
                            $filters[$attributeCode]['key'] = [$value['key']];
                        }

                        continue;
                    }

                    if (is_array($value['value'])) {
                        array_merge($filters[$attributeCode]['value'], $value['value']);
                        array_merge($filters[$attributeCode]['key'], $value['key']);
                        continue;
                    }

                    $filters[$attributeCode]['value'][] = $value['value'];
                    $filters[$attributeCode]['key'][] = $value['key'];
                }
            }

            $parentFilters = $refiloProduct->getData(Product::FILTERS);
            array_merge($parentFilters, $filters);
            $refiloProduct->setData(Product::FILTERS, array_merge($parentFilters, $filters));
            $refiloProduct->setVariants($variants);
        }

        $productOptions = null;
        return $refiloProduct;
    }

    /**
     * @param OptionInterface $option
     * @return array
     */
    protected function getOptions(OptionInterface $option, $usableAttributes)
    {
        $output = [];
        foreach ($option->getOptions() ?? [] as $optionValue) {

            if (!array_key_exists($optionValue['value_index'], $usableAttributes)) {
                continue;
            }

            $data = $this->productAttributeHelper->getSwatchByOptionId($optionValue['value_index']);
            $data['value_index'] = $optionValue['value_index'];
            $data['label'] = $optionValue['store_label'];
            $output[] = $data;
        }

        return $output;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getBundleChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        $extensionAttributes = $magentoProduct->getExtensionAttributes();
        $options = [];

        $refiloProduct->setData('bundle_price_type', (int)$magentoProduct->getPriceType() == 0 ? 'dynamic' : 'fixed');
        foreach ($extensionAttributes->getBundleProductOptions() as $bundleProductOption) {
            $productLinks = $bundleProductOption->getData('product_links');

            if (empty($productLinks)) {
                continue;
            }

            $option = [
                'id'       => (int)$bundleProductOption->getData('option_id'),
                'title'    => $bundleProductOption->getData('title'),
                'required' => (int)$bundleProductOption->getData('required'),
                'type'     => $bundleProductOption->getData('type'),
                'position' => (int)$bundleProductOption->getData('position'),
                'childs'   => []
            ];

            foreach ($productLinks as $productLink) {
                $link_data = $productLink->getData();
                $link_data['product_id'] = $link_data['entity_id'];
                $link_data['product_sku'] = $link_data['sku'];

                unset($link_data['entity_id']);
                unset($link_data['sku']);
                unset($link_data['option_id']);

                $child = [];
                foreach ($link_data as $key => $value) {
                    $child[$key] = (is_numeric($value)) ? $value + 0 : $value;
                }

                $option['childs'][] = $child;
            }

            $options[] = $option;
        }

        $refiloProduct->setBundle($options);

        return $refiloProduct;
    }

    /**
     * @param ProductInterface $magentoProduct
     * @param Product $refiloProduct
     * @return Product
     */
    public function getGroupedChildren(ProductInterface $magentoProduct, Product $refiloProduct)
    {
        $links = $magentoProduct->getTypeInstance()->getChildrenIds($magentoProduct->getId());

        if (empty($links)) {
            return $refiloProduct;
        }

        $refiloProduct->setChildren(array_values(array_shift($links)));
        return $refiloProduct;
    }
}
